<?php
/**
 * Plugin Name: Auto Post Publisher
 * Description: Manages scheduled posts and provides settings for custom post types.
 * Version: 1.3
 * Author: ExertLogics
 * Author URI: https://exertlogics.com/syed-gohar-ali/
 * Text Domain: auto-post-publisher
 * License: GPL-2.0-or-later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

class Auto_Post_Publisher {
    private $options;
    private const OPTION_NAME = 'auto_post_publisher_settings';
    private const CACHE_KEY = 'auto_publisher_missed_posts';
    private const CACHE_GROUP = 'auto_post_publisher';
    private const CACHE_EXPIRATION = 3600; // 1 hour

    public function __construct() {
        add_action('wp_head', [$this, 'publish_missed_posts']);
        add_action('admin_menu', [$this, 'add_settings_page']);
        add_action('admin_init', [$this, 'register_settings']);
        
        // Add cache clearing hooks
        add_action('save_post', [$this, 'clear_posts_cache']);
        add_action('delete_post', [$this, 'clear_posts_cache']);
        add_action('update_option_' . self::OPTION_NAME, [$this, 'clear_posts_cache']);
    }

    public function add_settings_page() {
        add_options_page(
            esc_html__('Auto Post Publisher Settings', 'auto-post-publisher'),
            esc_html__('Auto Publisher', 'auto-post-publisher'),
            'manage_options',
            'auto-post-publisher',
            [$this, 'render_settings_page']
        );
    }

    public function register_settings() {
        register_setting(
            'auto_post_publisher_settings',
            self::OPTION_NAME,
            [
                'type' => 'array',
                'description' => 'Auto Post Publisher Settings',
                'sanitize_callback' => [$this, 'sanitize_settings'],
                'show_in_rest' => false,
                'default' => [
                    'post_types' => ['post', 'page']
                ]
            ]
        );

        add_settings_section(
            'post_types_section',
            esc_html__('Post Types Settings', 'auto-post-publisher'),
            [$this, 'render_section_info'],
            'auto-post-publisher'
        );
    }

    public function sanitize_settings($input) {
        if (!is_array($input)) {
            return ['post_types' => ['post', 'page']];
        }

        $new_input = [];
        $allowed_post_types = array_keys(get_post_types(['public' => true]));
        
        if (isset($input['post_types']) && is_array($input['post_types'])) {
            $new_input['post_types'] = array_filter(
                array_map('sanitize_text_field', $input['post_types']),
                function($post_type) use ($allowed_post_types) {
                    return in_array($post_type, $allowed_post_types, true);
                }
            );
        }

        if (empty($new_input['post_types'])) {
            $new_input['post_types'] = ['post', 'page'];
        }
        
        return $new_input;
    }

    public function render_section_info() {
        echo wp_kses_post('<p>' . esc_html__('Select which post types to include in automatic post publishing:', 'auto-post-publisher') . '</p>');
    }

    public function render_settings_page() {
        if (!current_user_can('manage_options')) {
            wp_die(esc_html__('You do not have sufficient permissions to access this page.', 'auto-post-publisher'));
        }

        $post_types = get_post_types(['public' => true], 'objects');
        $this->options = get_option(self::OPTION_NAME);
        ?>
        <div class="wrap">
            <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
            <form method="post" action="options.php">
                <?php
                settings_fields('auto_post_publisher_settings');
                do_settings_sections('auto-post-publisher');
                ?>
                <table class="form-table" role="presentation">
                    <tr>
                        <th scope="row"><?php esc_html_e('Post Types', 'auto-post-publisher'); ?></th>
                        <td>
                            <?php foreach ($post_types as $post_type): ?>
                                <label>
                                    <input type="checkbox"
                                           name="<?php echo esc_attr(self::OPTION_NAME); ?>[post_types][]"
                                           value="<?php echo esc_attr($post_type->name); ?>"
                                           <?php checked(in_array($post_type->name, $this->options['post_types'] ?? [], true)); ?>>
                                    <?php echo esc_html($post_type->labels->name); ?>
                                </label><br>
                            <?php endforeach; ?>
                        </td>
                    </tr>
                </table>
                <?php submit_button(); ?>
            </form>
        </div>
        <?php
    }

    public function clear_posts_cache() {
        wp_cache_delete(self::CACHE_KEY, self::CACHE_GROUP);
    }

    public function get_missed_posts() {
        global $wpdb;
    
        // Try to get from cache first
        $cached_results = wp_cache_get(self::CACHE_KEY, self::CACHE_GROUP);
        if (false !== $cached_results) {
            return $cached_results;
        }
    
        $now = gmdate('Y-m-d H:i:00');
        $this->options = get_option(self::OPTION_NAME);
        $selected_post_types = $this->options['post_types'] ?? ['post', 'page'];
    
        // Build the placeholders for the post types
        $placeholders = implode(',', array_fill(0, count($selected_post_types), '%s'));
    
        // Build the query using $wpdb->prepare() for the entire query
        $results = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT ID FROM {$wpdb->posts} WHERE post_type IN ($placeholders) AND post_status = 'future' AND post_date_gmt < %s",
                array_merge($selected_post_types, [$now])
            )
        );
    
        // Cache the results
        wp_cache_set(self::CACHE_KEY, $results, self::CACHE_GROUP, self::CACHE_EXPIRATION);
    
        return $results;
    }
    

    public function publish_missed_posts() {
        if (!is_front_page() && !is_single()) {
            return;
        }

        $results = $this->get_missed_posts();
        if (!empty($results)) {
            foreach ($results as $result) {
                wp_publish_post($result->ID);
            }
            $this->clear_posts_cache();
        }
    }
}

// Initialize the plugin
new Auto_Post_Publisher();